@@ -1,24 +1,28 @@
-# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 from __future__ import with_statement
-from workbench.log import log_info, log_error, log_warning
+from workbench.log import log_info, log_error, log_warning, log_debug3
 
-from workbench.utils import format_duration
+from workbench.utils import format_duration, Version
 from workbench.db_utils import QueryError
 import mforms
 import time
@@ -26,12 +30,12 @@ import wb_admin_monitor
 
 def stradd(table, y, label, value):
     t = mforms.newLabel(label)
-    table.add(t, 0, 1, y, y+1, mforms.HFillFlag)
+    table.add(t, 0, 1, y, y+1, mforms.VFillFlag|mforms.HFillFlag)
 
     t = mforms.newLabel(value)
     t.set_style(mforms.BoldStyle)
     t.set_color("#555555")
-    table.add(t, 1, 2, y, y+1, mforms.HFillFlag)
+    table.add(t, 1, 2, y, y+1, mforms.VFillFlag|mforms.HFillFlag)
     return t
 
 
@@ -126,7 +130,6 @@ class ConnectionInfo(mforms.Box):
         self.info_table.set_row_count(8)
         self.info_table.set_column_spacing(18)
         self.info_table.set_row_spacing(5)
-        self.vbox.add(self.info_table, True, True)
 
         stradd(self.info_table, 0, "\nHost:", "\n"+info.get("hostname", "n/a"))
         stradd(self.info_table, 1, "Socket:", info.get("socket", "n/a"))
@@ -142,6 +145,7 @@ class ConnectionInfo(mforms.Box):
             stradd(self.info_table, 6, "Running Since:", "%s (%s)" % (time.ctime(ctrl_be.status_variables_time-uptime), format_duration(uptime, True)))
         else:
             stradd(self.info_table, 6, "Running Since:", "n/a")
+        self.vbox.add(self.info_table, True, True)
 
         box = mforms.newBox(True)
         refresh = mforms.newButton()
@@ -149,7 +153,7 @@ class ConnectionInfo(mforms.Box):
         refresh.set_tooltip("Refresh server status information")
         refresh.add_clicked_callback(self.owner.refresh_status)
         box.add(refresh, False, False)
-        self.info_table.add(box, 1, 2, 7, 8, 0)
+        self.info_table.add(box, 1, 2, 7, 8, mforms.VFillFlag)
 
         version = ctrl_be.target_version
         if version and info:
@@ -217,12 +221,13 @@ class WbAdminServerStatus(mforms.Box):
         self.status = wb_admin_monitor.WbAdminMonitor(server_profile, self.ctrl_be)
         self.status.set_size(360, -1)
         self.status.set_padding(0, 24, 24, 24)
-        self.add(self.status, False, False)
+        self.add(self.status, False, True)
 
         self.controls = {}
 
         self.currently_started = None
         self.ctrl_be.add_me_for_event("server_started", self)
+        self.ctrl_be.add_me_for_event("server_offline", self)
         self.ctrl_be.add_me_for_event("server_stopped", self)
 
         self.connection_info.update(self.ctrl_be)
@@ -235,6 +240,12 @@ class WbAdminServerStatus(mforms.Box):
             if not self._update_timeout:
                 self._update_timeout = mforms.Utilities.add_timeout(0.5, self.update)
 
+    def server_offline_event(self):
+        if self.currently_started != True:
+            self.refresh("offline")
+            self.currently_started = True
+            if not self._update_timeout:
+                self._update_timeout = mforms.Utilities.add_timeout(0.5, self.update)
     #---------------------------------------------------------------------------
     def server_stopped_event(self):
         if self.currently_started != False:
@@ -252,12 +263,12 @@ class WbAdminServerStatus(mforms.Box):
     def refresh_status(self):
         if not self._update_timeout:
             status = self.ctrl_be.force_check_server_state()
-            if status == "running" or not status and self.currently_started:
+            if (status == "running" or not status  or status == "offline" ) and self.currently_started:
                 self.ctrl_be.query_server_info()
 
             self._update_timeout = mforms.Utilities.add_timeout(0.5, self.update)
 
-    
+
     #---------------------------------------------------------------------------
     def page_activated(self):
         self.suspend_layout()
@@ -274,8 +285,14 @@ class WbAdminServerStatus(mforms.Box):
         if self.currently_started is None:
             if self.ctrl_be.is_server_running() == "running":
                 self.server_started_event()
+            elif self.ctrl_be.is_server_running() == "offline":
+                self.server_offline_event()
             else:
                 self.server_stopped_event()
+        else:
+            self.ctrl_be.query_server_info()
+
+        self.refresh(self.ctrl_be.is_server_running())
 
     def update(self):
         self._update_timeout = None
@@ -416,10 +433,10 @@ class WbAdminServerStatus(mforms.Box):
                                ("SSL Key:", lambda info, plugins, status: info.get("ssl_key") or "n/a")],
                               params)
 
-        log_error("mysql_firewall_trace: %s\n" % info.get("mysql_firewall_trace"))
-        log_error("Firewall_access_denied: %s\n" % status.get("Firewall_access_denied"))
-        log_error("Firewall_access_granted: %s\n" % status.get("Firewall_access_granted"))
-        log_error("Firewall_cached_entries: %s\n" % status.get("Firewall_cached_entries"))
+        log_debug3("mysql_firewall_trace: %s\n" % info.get("mysql_firewall_trace"))
+        log_debug3("Firewall_access_denied: %s\n" % status.get("Firewall_access_denied"))
+        log_debug3("Firewall_access_granted: %s\n" % status.get("Firewall_access_granted"))
+        log_debug3("Firewall_cached_entries: %s\n" % status.get("Firewall_cached_entries"))
 
         if info.get("mysql_firewall_mode") == "ON":
             self.add_info_section("Firewall",
@@ -447,6 +464,7 @@ class WbAdminServerStatus(mforms.Box):
         hbox.add(info_table, True, True)
 
         self.content.add(hbox, False, True)
+        self.content.get_parent().relayout()
 
 
     def add_info_section(self, title, info, params):
@@ -462,6 +480,7 @@ class WbAdminServerStatus(mforms.Box):
         info_table = self.make_info_table([x for x in info if x], params)
         self.content.add(info_table, False, True)
         self.controls[title] = (info_table, None)
+        self.content.get_parent().relayout()
 
 
     def make_info_table(self, info, params):
@@ -484,26 +503,34 @@ class WbAdminServerStatus(mforms.Box):
             if self.controls.has_key(label):
                 info_table.remove(self.controls[label][0])
             else:
-                info_table.add(mforms.newLabel(label), 0, 1, i, i+1, mforms.HFillFlag)
-
+                info_table.add(mforms.newLabel(label), 0, 1, i, i+1, mforms.VFillFlag|mforms.HFillFlag)
+            is_gtid_mode_setable = label == 'GTID Mode:' and self.ctrl_be.target_version >= Version(5, 7, 6)
             if type(value) is bool or value is None:
                 b = StateIcon()
                 b.set_state(value)
-                info_table.add(b, 1, 2, i, i+1, mforms.HFillFlag|mforms.HExpandFlag)
+                info_table.add(b, 1, 2, i, i+1, mforms.HFillFlag | mforms.HExpandFlag | mforms.VFillFlag)
                 self.controls[label] = (b, value_source)
             elif type(value) is tuple:
                 b = StateIcon()
                 b.set_state(value[0])
                 if value[0] and value[1]:
                     b.set_text(value[1])
-                info_table.add(b, 1, 2, i, i+1, mforms.HFillFlag|mforms.HExpandFlag)
+                info_table.add(b, 1, 2, i, i+1, mforms.HFillFlag | mforms.HExpandFlag | mforms.VFillFlag)
                 self.controls[label] = (b, value_source)
             else:
-                l2 = mforms.newLabel(value or "")
-                l2.set_style(mforms.BoldStyle)
-                l2.set_color("#1c1c1c")
-                info_table.add(l2, 1, 2, i, i+1, mforms.HFillFlag|mforms.HExpandFlag)
-                self.controls[label] = (l2, value_source)
+                if is_gtid_mode_setable:
+                    self.gtid_mode_selector = mforms.newSelector()
+                    self.gtid_mode_selector.add_items(["OFF", "UPGRADE_STEP_1", "UPGRADE_STEP_1", "ON"])
+                    self.gtid_mode_selector.set_selected(self.gtid_mode_selector.index_of_item_with_title(value_source))
+                    self.gtid_mode_selector.add_changed_callback(self._gtid_mode_changed)
+                    info_table.add(self.gtid_mode_selector, 1, 2, i, i + 1, mforms.HFillFlag | mforms.HExpandFlag | mforms.VFillFlag)
+                    self.controls[label] = (self.gtid_mode_selector, value_source)
+                else:
+                    l2 = mforms.newLabel(value or "")
+                    l2.set_style(mforms.BoldStyle)
+                    l2.set_color("#1c1c1c")
+                    info_table.add(l2, 1, 2, i, i + 1, mforms.HFillFlag | mforms.HExpandFlag | mforms.VFillFlag)
+                    self.controls[label] = (l2, value_source)
         info_table.add(mforms.newLabel(""), 0, 1, len(info), len(info)+1, mforms.HFillFlag) # blank space
         return info_table
 
@@ -519,3 +546,10 @@ class WbAdminServerStatus(mforms.Box):
             self._update_timeout = None
         self.status.stop()
 
+    #---------------------------------------------------------------------------
+    def _gtid_mode_changed(self):
+        new_value = self.gtid_mode_selector.get_string_value()
+        try:
+            self.ctrl_be.exec_query("SET @@GLOBAL.GTID_MODE = %s;" % new_value)
+        except QueryError, e:
+            log_error("Error update GTID mode: %s" % str(e))
